@@ -6,10 +6,22 @@ import hashlib
 import logging
 from abc import ABC, abstractmethod
 from pathlib import Path
-from typing import Callable, Literal
+from typing import Callable, Literal, TypeAlias
 
 import anomed_utils as utils
 import numpy as np
+import pandas as pd
+
+__all__ = [
+    "InMemoryNumpyArrays",
+    "NpzFromDisk",
+    "NumpyDataset",
+    "SupervisedLearningMIAChallenge",
+    "discard_targets",
+    "evaluate_membership_inference_attack",
+    "evaluate_MIA",
+    "strict_binary_accuracy",
+]
 
 _logger = logging.getLogger(__name__)
 
@@ -100,10 +112,18 @@ class InMemoryNumpyArrays(NumpyDataset):
     memory."""
 
     def __init__(self, X: np.ndarray, y: np.ndarray):
+        """
+        Parameters
+        ----------
+        X : np.ndarray
+            The feature array.
+        y : np.ndarray
+            The target array.
+        """
         self._X = X
         self._y = y
 
-    def get(self):
+    def get(self) -> tuple[np.ndarray, np.ndarray]:
         return (self._X, self._y)
 
 
@@ -523,7 +543,9 @@ def strict_binary_accuracy(
     the ground truth.
 
     By strict accuracy, we mean the fraction of the number of times where
+
         `prediction[i] == ground_truth[i]`,
+
     for `0 <= i < len(prediction)` is `True`, divided by `len(prediction)`. Note
     this is not the same as `sum(prediction==ground_truth)`, which is more
     forgiving for higher dimensional arrays (for one-dimensional arrays, it is
@@ -621,3 +643,75 @@ def evaluate_MIA(prediction: np.ndarray, ground_truth: np.ndarray) -> dict[str,
     return evaluate_membership_inference_attack(
         prediction=prediction, ground_truth=ground_truth
     )
+
+
+AnonymizationScheme: TypeAlias = Literal["same", "generalization", "microaggregation"]
+"""What scheme does the anonymized data follow?
+
+- same: Leaky data and anonymized data match regarding shape (columns) and
+  datatype.
+- generalization: Cardinal data will be represented by (min, max) ranges,
+  nominal data will be one-hot encoded.
+- microaggregation: TODO
+"""
+
+
+class TabularDataResconstructionChallenge:
+    """This class represents challenges that aim to protect tabular data via
+    anonymization, while maintaining certain utility. The threat model involves
+    attackers that aim to reconstruct the tabular data from the anonymized data
+    plus background knowledge."""
+
+    def __init__(
+        self,
+        leaky_data: pd.DataFrame,
+        background_knowledge: pd.DataFrame,
+        utility_evaluator: Callable[
+            [pd.DataFrame, AnonymizationScheme, pd.DataFrame], dict[str, float]
+        ],
+        privacy_evaluator: Callable[[pd.DataFrame, pd.DataFrame], dict[str, float]],
+    ):
+        """
+        Parameters
+        ----------
+        leaky_data : pd.Dataframe
+            The original tabular data which should be protected by
+            anonymization.
+        background_knowledge : pd.DataFrame
+            Background knowledge that is accessible by attackers.
+        utility_evaluator : Callable[[pd.DataFrame, AnonymizationScheme, DataFrame], dict[str, float]]
+            How to evaluate the utility of anonymized data, depending of the
+            kind of anonymization. The first argument is the anonymized data,
+            the second argument is the anonymization scheme and the third
+            argument is the original/leaky data for comparison.
+
+            This function parameter is used, when `evaluate_utility` is called.
+        privacy_evaluator : Callable[[pd.DataFrame, pd.DataFrame], dict[str, float]]
+            How to evaluate the quality of reconstructed data, i.e. the success
+            of the attacker. This is also an indirect measure of how privacy
+            preserving the anonymized data (as an attack target) has been.
+            The first argument is the reconstructed data, the second argument
+            the leaky data, which the attack target's anonymized data depends
+            on.
+
+            This function parameter is used, when `evaluate_privacy` is called.
+        """
+        self._leaky_data = leaky_data
+        self._background_knowledge = background_knowledge
+        self._evaluate_utility = utility_evaluator
+        self._evaluate_privacy = privacy_evaluator
+
+    def evaluate_utility(
+        self,
+        anonymized_data: pd.DataFrame,
+        anonymization_scheme: AnonymizationScheme,
+        leaky_data: pd.DataFrame,
+    ) -> dict[str, float]:
+        return self._evaluate_utility(anonymized_data, anonymization_scheme, leaky_data)
+
+    def evaluate_privacy(
+        self,
+        reconstructed_data: pd.DataFrame,
+        leaky_data: pd.DataFrame,
+    ) -> dict[str, float]:
+        return self._evaluate_privacy(reconstructed_data, leaky_data)
